# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from lumaai import LumaAI, AsyncLumaAI
from tests.utils import assert_matches_type
from lumaai.types import (
    Generation,
    GenerationListResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestGenerations:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: LumaAI) -> None:
        generation = client.generations.create(
            model="ray-1-6",
        )
        assert_matches_type(Generation, generation, path=["response"])

    @parametrize
    def test_method_create_with_all_params(self, client: LumaAI) -> None:
        generation = client.generations.create(
            model="ray-1-6",
            aspect_ratio="16:9",
            callback_url="https://example.com",
            concepts=[{"key": "key"}],
            duration="5s",
            generation_type="video",
            keyframes={
                "frame0": {
                    "type": "image",
                    "url": "https://example.com/image.jpg",
                },
                "frame1": {
                    "id": "123e4567-e89b-12d3-a456-426614174000",
                    "type": "generation",
                },
            },
            loop=True,
            prompt="A serene lake surrounded by mountains at sunset",
            resolution="540p",
        )
        assert_matches_type(Generation, generation, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: LumaAI) -> None:
        response = client.generations.with_raw_response.create(
            model="ray-1-6",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        generation = response.parse()
        assert_matches_type(Generation, generation, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: LumaAI) -> None:
        with client.generations.with_streaming_response.create(
            model="ray-1-6",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            generation = response.parse()
            assert_matches_type(Generation, generation, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_list(self, client: LumaAI) -> None:
        generation = client.generations.list()
        assert_matches_type(GenerationListResponse, generation, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: LumaAI) -> None:
        generation = client.generations.list(
            limit=0,
            offset=0,
        )
        assert_matches_type(GenerationListResponse, generation, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: LumaAI) -> None:
        response = client.generations.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        generation = response.parse()
        assert_matches_type(GenerationListResponse, generation, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: LumaAI) -> None:
        with client.generations.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            generation = response.parse()
            assert_matches_type(GenerationListResponse, generation, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_delete(self, client: LumaAI) -> None:
        generation = client.generations.delete(
            "id",
        )
        assert generation is None

    @parametrize
    def test_raw_response_delete(self, client: LumaAI) -> None:
        response = client.generations.with_raw_response.delete(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        generation = response.parse()
        assert generation is None

    @parametrize
    def test_streaming_response_delete(self, client: LumaAI) -> None:
        with client.generations.with_streaming_response.delete(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            generation = response.parse()
            assert generation is None

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_delete(self, client: LumaAI) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.generations.with_raw_response.delete(
                "",
            )

    @parametrize
    def test_method_audio(self, client: LumaAI) -> None:
        generation = client.generations.audio(
            id="id",
        )
        assert_matches_type(Generation, generation, path=["response"])

    @parametrize
    def test_method_audio_with_all_params(self, client: LumaAI) -> None:
        generation = client.generations.audio(
            id="id",
            callback_url="https://example.com",
            generation_type="add_audio",
            negative_prompt="negative_prompt",
            prompt="prompt",
        )
        assert_matches_type(Generation, generation, path=["response"])

    @parametrize
    def test_raw_response_audio(self, client: LumaAI) -> None:
        response = client.generations.with_raw_response.audio(
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        generation = response.parse()
        assert_matches_type(Generation, generation, path=["response"])

    @parametrize
    def test_streaming_response_audio(self, client: LumaAI) -> None:
        with client.generations.with_streaming_response.audio(
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            generation = response.parse()
            assert_matches_type(Generation, generation, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_audio(self, client: LumaAI) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.generations.with_raw_response.audio(
                id="",
            )

    @parametrize
    def test_method_get(self, client: LumaAI) -> None:
        generation = client.generations.get(
            "id",
        )
        assert_matches_type(Generation, generation, path=["response"])

    @parametrize
    def test_raw_response_get(self, client: LumaAI) -> None:
        response = client.generations.with_raw_response.get(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        generation = response.parse()
        assert_matches_type(Generation, generation, path=["response"])

    @parametrize
    def test_streaming_response_get(self, client: LumaAI) -> None:
        with client.generations.with_streaming_response.get(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            generation = response.parse()
            assert_matches_type(Generation, generation, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_get(self, client: LumaAI) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.generations.with_raw_response.get(
                "",
            )

    @parametrize
    def test_method_upscale(self, client: LumaAI) -> None:
        generation = client.generations.upscale(
            id="id",
        )
        assert_matches_type(Generation, generation, path=["response"])

    @parametrize
    def test_method_upscale_with_all_params(self, client: LumaAI) -> None:
        generation = client.generations.upscale(
            id="id",
            callback_url="https://example.com",
            generation_type="upscale_video",
            resolution="540p",
        )
        assert_matches_type(Generation, generation, path=["response"])

    @parametrize
    def test_raw_response_upscale(self, client: LumaAI) -> None:
        response = client.generations.with_raw_response.upscale(
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        generation = response.parse()
        assert_matches_type(Generation, generation, path=["response"])

    @parametrize
    def test_streaming_response_upscale(self, client: LumaAI) -> None:
        with client.generations.with_streaming_response.upscale(
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            generation = response.parse()
            assert_matches_type(Generation, generation, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_upscale(self, client: LumaAI) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.generations.with_raw_response.upscale(
                id="",
            )


class TestAsyncGenerations:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_create(self, async_client: AsyncLumaAI) -> None:
        generation = await async_client.generations.create(
            model="ray-1-6",
        )
        assert_matches_type(Generation, generation, path=["response"])

    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncLumaAI) -> None:
        generation = await async_client.generations.create(
            model="ray-1-6",
            aspect_ratio="16:9",
            callback_url="https://example.com",
            concepts=[{"key": "key"}],
            duration="5s",
            generation_type="video",
            keyframes={
                "frame0": {
                    "type": "image",
                    "url": "https://example.com/image.jpg",
                },
                "frame1": {
                    "id": "123e4567-e89b-12d3-a456-426614174000",
                    "type": "generation",
                },
            },
            loop=True,
            prompt="A serene lake surrounded by mountains at sunset",
            resolution="540p",
        )
        assert_matches_type(Generation, generation, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncLumaAI) -> None:
        response = await async_client.generations.with_raw_response.create(
            model="ray-1-6",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        generation = await response.parse()
        assert_matches_type(Generation, generation, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncLumaAI) -> None:
        async with async_client.generations.with_streaming_response.create(
            model="ray-1-6",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            generation = await response.parse()
            assert_matches_type(Generation, generation, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_list(self, async_client: AsyncLumaAI) -> None:
        generation = await async_client.generations.list()
        assert_matches_type(GenerationListResponse, generation, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncLumaAI) -> None:
        generation = await async_client.generations.list(
            limit=0,
            offset=0,
        )
        assert_matches_type(GenerationListResponse, generation, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncLumaAI) -> None:
        response = await async_client.generations.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        generation = await response.parse()
        assert_matches_type(GenerationListResponse, generation, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncLumaAI) -> None:
        async with async_client.generations.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            generation = await response.parse()
            assert_matches_type(GenerationListResponse, generation, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_delete(self, async_client: AsyncLumaAI) -> None:
        generation = await async_client.generations.delete(
            "id",
        )
        assert generation is None

    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncLumaAI) -> None:
        response = await async_client.generations.with_raw_response.delete(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        generation = await response.parse()
        assert generation is None

    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncLumaAI) -> None:
        async with async_client.generations.with_streaming_response.delete(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            generation = await response.parse()
            assert generation is None

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_delete(self, async_client: AsyncLumaAI) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.generations.with_raw_response.delete(
                "",
            )

    @parametrize
    async def test_method_audio(self, async_client: AsyncLumaAI) -> None:
        generation = await async_client.generations.audio(
            id="id",
        )
        assert_matches_type(Generation, generation, path=["response"])

    @parametrize
    async def test_method_audio_with_all_params(self, async_client: AsyncLumaAI) -> None:
        generation = await async_client.generations.audio(
            id="id",
            callback_url="https://example.com",
            generation_type="add_audio",
            negative_prompt="negative_prompt",
            prompt="prompt",
        )
        assert_matches_type(Generation, generation, path=["response"])

    @parametrize
    async def test_raw_response_audio(self, async_client: AsyncLumaAI) -> None:
        response = await async_client.generations.with_raw_response.audio(
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        generation = await response.parse()
        assert_matches_type(Generation, generation, path=["response"])

    @parametrize
    async def test_streaming_response_audio(self, async_client: AsyncLumaAI) -> None:
        async with async_client.generations.with_streaming_response.audio(
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            generation = await response.parse()
            assert_matches_type(Generation, generation, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_audio(self, async_client: AsyncLumaAI) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.generations.with_raw_response.audio(
                id="",
            )

    @parametrize
    async def test_method_get(self, async_client: AsyncLumaAI) -> None:
        generation = await async_client.generations.get(
            "id",
        )
        assert_matches_type(Generation, generation, path=["response"])

    @parametrize
    async def test_raw_response_get(self, async_client: AsyncLumaAI) -> None:
        response = await async_client.generations.with_raw_response.get(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        generation = await response.parse()
        assert_matches_type(Generation, generation, path=["response"])

    @parametrize
    async def test_streaming_response_get(self, async_client: AsyncLumaAI) -> None:
        async with async_client.generations.with_streaming_response.get(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            generation = await response.parse()
            assert_matches_type(Generation, generation, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_get(self, async_client: AsyncLumaAI) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.generations.with_raw_response.get(
                "",
            )

    @parametrize
    async def test_method_upscale(self, async_client: AsyncLumaAI) -> None:
        generation = await async_client.generations.upscale(
            id="id",
        )
        assert_matches_type(Generation, generation, path=["response"])

    @parametrize
    async def test_method_upscale_with_all_params(self, async_client: AsyncLumaAI) -> None:
        generation = await async_client.generations.upscale(
            id="id",
            callback_url="https://example.com",
            generation_type="upscale_video",
            resolution="540p",
        )
        assert_matches_type(Generation, generation, path=["response"])

    @parametrize
    async def test_raw_response_upscale(self, async_client: AsyncLumaAI) -> None:
        response = await async_client.generations.with_raw_response.upscale(
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        generation = await response.parse()
        assert_matches_type(Generation, generation, path=["response"])

    @parametrize
    async def test_streaming_response_upscale(self, async_client: AsyncLumaAI) -> None:
        async with async_client.generations.with_streaming_response.upscale(
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            generation = await response.parse()
            assert_matches_type(Generation, generation, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_upscale(self, async_client: AsyncLumaAI) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.generations.with_raw_response.upscale(
                id="",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union, Optional
from datetime import datetime
from typing_extensions import Literal, Annotated, TypeAlias

from .._utils import PropertyInfo
from .._models import BaseModel

__all__ = [
    "Generation",
    "Assets",
    "Request",
    "RequestGenerationRequest",
    "RequestGenerationRequestConcept",
    "RequestGenerationRequestKeyframes",
    "RequestGenerationRequestKeyframesFrame0",
    "RequestGenerationRequestKeyframesFrame0GenerationReference",
    "RequestGenerationRequestKeyframesFrame0ImageReference",
    "RequestGenerationRequestKeyframesFrame1",
    "RequestGenerationRequestKeyframesFrame1GenerationReference",
    "RequestGenerationRequestKeyframesFrame1ImageReference",
    "RequestImageGenerationRequest",
    "RequestImageGenerationRequestCharacterRef",
    "RequestImageGenerationRequestCharacterRefIdentity0",
    "RequestImageGenerationRequestImageRef",
    "RequestImageGenerationRequestModifyImageRef",
    "RequestImageGenerationRequestStyleRef",
    "RequestUpscaleVideoGenerationRequest",
    "RequestAudioGenerationRequest",
    "RequestReframeImageRequest",
    "RequestReframeImageRequestMedia",
    "RequestReframeVideoRequest",
    "RequestReframeVideoRequestMedia",
    "RequestReframeVideoRequestFirstFrame",
    "RequestModifyVideoRequest",
    "RequestModifyVideoRequestMedia",
    "RequestModifyVideoRequestFirstFrame",
]


class Assets(BaseModel):
    image: Optional[str] = None
    """The URL of the image"""

    progress_video: Optional[str] = None
    """The URL of the progress video"""

    video: Optional[str] = None
    """The URL of the video"""


class RequestGenerationRequestConcept(BaseModel):
    key: str
    """The key of the concept"""


class RequestGenerationRequestKeyframesFrame0GenerationReference(BaseModel):
    id: str
    """The ID of the generation"""

    type: Literal["generation"]


class RequestGenerationRequestKeyframesFrame0ImageReference(BaseModel):
    type: Literal["image"]

    url: str
    """The URL of the image"""


RequestGenerationRequestKeyframesFrame0: TypeAlias = Annotated[
    Union[
        RequestGenerationRequestKeyframesFrame0GenerationReference,
        RequestGenerationRequestKeyframesFrame0ImageReference,
    ],
    PropertyInfo(discriminator="type"),
]


class RequestGenerationRequestKeyframesFrame1GenerationReference(BaseModel):
    id: str
    """The ID of the generation"""

    type: Literal["generation"]


class RequestGenerationRequestKeyframesFrame1ImageReference(BaseModel):
    type: Literal["image"]

    url: str
    """The URL of the image"""


RequestGenerationRequestKeyframesFrame1: TypeAlias = Annotated[
    Union[
        RequestGenerationRequestKeyframesFrame1GenerationReference,
        RequestGenerationRequestKeyframesFrame1ImageReference,
    ],
    PropertyInfo(discriminator="type"),
]


class RequestGenerationRequestKeyframes(BaseModel):
    frame0: Optional[RequestGenerationRequestKeyframesFrame0] = None
    """The frame 0 of the generation"""

    frame1: Optional[RequestGenerationRequestKeyframesFrame1] = None
    """The frame 1 of the generation"""


class RequestGenerationRequest(BaseModel):
    model: Literal["ray-1-6", "ray-2", "ray-flash-2"]
    """The model used for the generation"""

    aspect_ratio: Optional[Literal["1:1", "16:9", "9:16", "4:3", "3:4", "21:9", "9:21"]] = None
    """The aspect ratio of the generation"""

    callback_url: Optional[str] = None
    """
    The callback URL of the generation, a POST request with Generation object will
    be sent to the callback URL when the generation is dreaming, completed, or
    failed
    """

    concepts: Optional[List[RequestGenerationRequestConcept]] = None
    """The concepts of the generation"""

    duration: Union[Literal["5s", "9s"], str, None] = None
    """The duration of the generation"""

    generation_type: Optional[Literal["video"]] = None

    keyframes: Optional[RequestGenerationRequestKeyframes] = None
    """The keyframes of the generation"""

    loop: Optional[bool] = None
    """Whether to loop the video"""

    prompt: Optional[str] = None
    """The prompt of the generation"""

    resolution: Union[Literal["540p", "720p", "1080p", "4k"], str, None] = None
    """The resolution of the generation"""


class RequestImageGenerationRequestCharacterRefIdentity0(BaseModel):
    images: Optional[List[str]] = None
    """The URLs of the image identity"""


class RequestImageGenerationRequestCharacterRef(BaseModel):
    identity0: Optional[RequestImageGenerationRequestCharacterRefIdentity0] = None
    """The image identity object"""


class RequestImageGenerationRequestImageRef(BaseModel):
    url: Optional[str] = None
    """The URL of the image reference"""

    weight: Optional[float] = None
    """The weight of the image reference"""


class RequestImageGenerationRequestModifyImageRef(BaseModel):
    url: Optional[str] = None
    """The URL of the image reference"""

    weight: Optional[float] = None
    """The weight of the modify image reference"""


class RequestImageGenerationRequestStyleRef(BaseModel):
    url: Optional[str] = None
    """The URL of the image reference"""

    weight: Optional[float] = None
    """The weight of the image reference"""


class RequestImageGenerationRequest(BaseModel):
    model: Literal["photon-1", "photon-flash-1"]
    """The model used for the generation"""

    aspect_ratio: Optional[Literal["1:1", "16:9", "9:16", "4:3", "3:4", "21:9", "9:21"]] = None
    """The aspect ratio of the generation"""

    callback_url: Optional[str] = None
    """The callback URL for the generation"""

    character_ref: Optional[RequestImageGenerationRequestCharacterRef] = None

    format: Optional[Literal["jpg", "png"]] = None
    """The format of the image"""

    generation_type: Optional[Literal["image"]] = None

    image_ref: Optional[List[RequestImageGenerationRequestImageRef]] = None

    modify_image_ref: Optional[RequestImageGenerationRequestModifyImageRef] = None
    """The modify image reference object"""

    prompt: Optional[str] = None
    """The prompt of the generation"""

    style_ref: Optional[List[RequestImageGenerationRequestStyleRef]] = None

    sync: Optional[bool] = None
    """Create image in synchronous mode and return complated image"""

    sync_timeout: Optional[float] = None
    """The timeout for the synchronous image generation"""


class RequestUpscaleVideoGenerationRequest(BaseModel):
    callback_url: Optional[str] = None
    """The callback URL for the upscale"""

    generation_type: Optional[Literal["upscale_video"]] = None

    resolution: Union[Literal["540p", "720p", "1080p", "4k"], str, None] = None
    """The resolution of the upscale"""


class RequestAudioGenerationRequest(BaseModel):
    callback_url: Optional[str] = None
    """The callback URL for the audio"""

    generation_type: Optional[Literal["add_audio"]] = None

    negative_prompt: Optional[str] = None
    """The negative prompt of the audio"""

    prompt: Optional[str] = None
    """The prompt of the audio"""


class RequestReframeImageRequestMedia(BaseModel):
    url: str
    """The URL of the media"""


class RequestReframeImageRequest(BaseModel):
    aspect_ratio: Literal["1:1", "16:9", "9:16", "4:3", "3:4", "21:9", "9:21"]
    """The aspect ratio of the generation"""

    generation_type: Literal["reframe_image"]

    media: RequestReframeImageRequestMedia
    """The image entity object"""

    model: Literal["photon-1", "photon-flash-1"]
    """The model used for the reframe image"""

    callback_url: Optional[str] = None
    """
    The callback URL of the generation, a POST request with Generation object will
    be sent to the callback URL when the generation is dreaming, completed, or
    failed
    """

    format: Optional[Literal["jpg", "png"]] = None
    """The format of the image"""

    grid_position_x: Optional[int] = None
    """The x position of the image in the grid"""

    grid_position_y: Optional[int] = None
    """The y position of the image in the grid"""

    prompt: Optional[str] = None
    """The prompt of the generation"""

    resized_height: Optional[int] = None
    """Resized height of source image"""

    resized_width: Optional[int] = None
    """Resized width of source image"""

    x_end: Optional[int] = None
    """The x end of the crop bounds"""

    x_start: Optional[int] = None
    """The x start of the crop bounds"""

    y_end: Optional[int] = None
    """The y end of the crop bounds"""

    y_start: Optional[int] = None
    """The y start of the crop bounds"""


class RequestReframeVideoRequestMedia(BaseModel):
    url: str
    """The URL of the media"""


class RequestReframeVideoRequestFirstFrame(BaseModel):
    url: str
    """The URL of the media"""


class RequestReframeVideoRequest(BaseModel):
    aspect_ratio: Literal["1:1", "16:9", "9:16", "4:3", "3:4", "21:9", "9:21"]
    """The aspect ratio of the generation"""

    generation_type: Literal["reframe_video"]

    media: RequestReframeVideoRequestMedia
    """The image entity object"""

    model: Literal["ray-2", "ray-flash-2"]
    """The model used for the reframe video"""

    callback_url: Optional[str] = None
    """
    The callback URL of the generation, a POST request with Generation object will
    be sent to the callback URL when the generation is dreaming, completed, or
    failed
    """

    first_frame: Optional[RequestReframeVideoRequestFirstFrame] = None
    """The image entity object"""

    grid_position_x: Optional[int] = None
    """The x position of the image in the grid"""

    grid_position_y: Optional[int] = None
    """The y position of the image in the grid"""

    prompt: Optional[str] = None
    """The prompt of the generation"""

    resized_height: Optional[int] = None
    """Resized height of source video"""

    resized_width: Optional[int] = None
    """Resized width of source video"""

    x_end: Optional[int] = None
    """The x end of the crop bounds"""

    x_start: Optional[int] = None
    """The x start of the crop bounds"""

    y_end: Optional[int] = None
    """The y end of the crop bounds"""

    y_start: Optional[int] = None
    """The y start of the crop bounds"""


class RequestModifyVideoRequestMedia(BaseModel):
    url: str
    """The URL of the media"""


class RequestModifyVideoRequestFirstFrame(BaseModel):
    url: str
    """The URL of the media"""


class RequestModifyVideoRequest(BaseModel):
    generation_type: Literal["modify_video"]

    media: RequestModifyVideoRequestMedia
    """The image entity object"""

    mode: Literal[
        "adhere_1", "adhere_2", "adhere_3", "flex_1", "flex_2", "flex_3", "reimagine_1", "reimagine_2", "reimagine_3"
    ]
    """The mode of the modify video"""

    model: Literal["ray-2"]
    """The model used for the modify video"""

    callback_url: Optional[str] = None
    """
    The callback URL of the generation, a POST request with Generation object will
    be sent to the callback URL when the generation is dreaming, completed, or
    failed
    """

    first_frame: Optional[RequestModifyVideoRequestFirstFrame] = None
    """The image entity object"""

    prompt: Optional[str] = None
    """The prompt of the generation"""


Request: TypeAlias = Union[
    RequestGenerationRequest,
    RequestImageGenerationRequest,
    RequestUpscaleVideoGenerationRequest,
    RequestAudioGenerationRequest,
    RequestReframeImageRequest,
    RequestReframeVideoRequest,
    RequestModifyVideoRequest,
]


class Generation(BaseModel):
    id: Optional[str] = None
    """The ID of the generation"""

    assets: Optional[Assets] = None
    """The assets of the generation"""

    created_at: Optional[datetime] = None
    """The date and time when the generation was created"""

    failure_reason: Optional[str] = None
    """The reason for the state of the generation"""

    generation_type: Optional[Literal["video", "image"]] = None
    """The type of the generation"""

    model: Optional[str] = None
    """The model used for the generation"""

    request: Optional[Request] = None
    """The request of the generation"""

    state: Optional[Literal["queued", "dreaming", "completed", "failed"]] = None
    """The state of the generation"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable
from typing_extensions import Literal, Required, TypeAlias, TypedDict

__all__ = [
    "GenerationCreateParams",
    "Concept",
    "Keyframes",
    "KeyframesFrame0",
    "KeyframesFrame0GenerationReference",
    "KeyframesFrame0ImageReference",
    "KeyframesFrame1",
    "KeyframesFrame1GenerationReference",
    "KeyframesFrame1ImageReference",
]


class GenerationCreateParams(TypedDict, total=False):
    model: Required[Literal["ray-1-6", "ray-2", "ray-flash-2"]]
    """The model used for the generation"""

    aspect_ratio: Literal["1:1", "16:9", "9:16", "4:3", "3:4", "21:9", "9:21"]
    """The aspect ratio of the generation"""

    callback_url: str
    """
    The callback URL of the generation, a POST request with Generation object will
    be sent to the callback URL when the generation is dreaming, completed, or
    failed
    """

    concepts: Iterable[Concept]
    """The concepts of the generation"""

    duration: Union[Literal["5s", "9s"], str]
    """The duration of the generation"""

    generation_type: Literal["video"]

    keyframes: Keyframes
    """The keyframes of the generation"""

    loop: bool
    """Whether to loop the video"""

    prompt: str
    """The prompt of the generation"""

    resolution: Union[Literal["540p", "720p", "1080p", "4k"], str]
    """The resolution of the generation"""


class Concept(TypedDict, total=False):
    key: Required[str]
    """The key of the concept"""


class KeyframesFrame0GenerationReference(TypedDict, total=False):
    id: Required[str]
    """The ID of the generation"""

    type: Required[Literal["generation"]]


class KeyframesFrame0ImageReference(TypedDict, total=False):
    type: Required[Literal["image"]]

    url: Required[str]
    """The URL of the image"""


KeyframesFrame0: TypeAlias = Union[KeyframesFrame0GenerationReference, KeyframesFrame0ImageReference]


class KeyframesFrame1GenerationReference(TypedDict, total=False):
    id: Required[str]
    """The ID of the generation"""

    type: Required[Literal["generation"]]


class KeyframesFrame1ImageReference(TypedDict, total=False):
    type: Required[Literal["image"]]

    url: Required[str]
    """The URL of the image"""


KeyframesFrame1: TypeAlias = Union[KeyframesFrame1GenerationReference, KeyframesFrame1ImageReference]


class Keyframes(TypedDict, total=False):
    frame0: KeyframesFrame0
    """The frame 0 of the generation"""

    frame1: KeyframesFrame1
    """The frame 1 of the generation"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Literal

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.generation import Generation
from ...types.generations import image_create_params, image_reframe_params

__all__ = ["ImageResource", "AsyncImageResource"]


class ImageResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ImageResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/lumalabs/lumaai-python#accessing-raw-response-data-eg-headers
        """
        return ImageResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ImageResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/lumalabs/lumaai-python#with_streaming_response
        """
        return ImageResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        model: Literal["photon-1", "photon-flash-1"],
        aspect_ratio: Literal["1:1", "16:9", "9:16", "4:3", "3:4", "21:9", "9:21"] | NotGiven = NOT_GIVEN,
        callback_url: str | NotGiven = NOT_GIVEN,
        character_ref: image_create_params.CharacterRef | NotGiven = NOT_GIVEN,
        format: Literal["jpg", "png"] | NotGiven = NOT_GIVEN,
        generation_type: Literal["image"] | NotGiven = NOT_GIVEN,
        image_ref: Iterable[image_create_params.ImageRef] | NotGiven = NOT_GIVEN,
        modify_image_ref: image_create_params.ModifyImageRef | NotGiven = NOT_GIVEN,
        prompt: str | NotGiven = NOT_GIVEN,
        style_ref: Iterable[image_create_params.StyleRef] | NotGiven = NOT_GIVEN,
        sync: bool | NotGiven = NOT_GIVEN,
        sync_timeout: float | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Generation:
        """
        Generate an image with the provided prompt

        Args:
          model: The model used for the generation

          aspect_ratio: The aspect ratio of the generation

          callback_url: The callback URL for the generation

          format: The format of the image

          modify_image_ref: The modify image reference object

          prompt: The prompt of the generation

          sync: Create image in synchronous mode and return complated image

          sync_timeout: The timeout for the synchronous image generation

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/generations/image",
            body=maybe_transform(
                {
                    "model": model,
                    "aspect_ratio": aspect_ratio,
                    "callback_url": callback_url,
                    "character_ref": character_ref,
                    "format": format,
                    "generation_type": generation_type,
                    "image_ref": image_ref,
                    "modify_image_ref": modify_image_ref,
                    "prompt": prompt,
                    "style_ref": style_ref,
                    "sync": sync,
                    "sync_timeout": sync_timeout,
                },
                image_create_params.ImageCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Generation,
        )

    def reframe(
        self,
        *,
        aspect_ratio: Literal["1:1", "16:9", "9:16", "4:3", "3:4", "21:9", "9:21"],
        generation_type: Literal["reframe_image"],
        media: image_reframe_params.Media,
        model: Literal["photon-1", "photon-flash-1"],
        callback_url: str | NotGiven = NOT_GIVEN,
        format: Literal["jpg", "png"] | NotGiven = NOT_GIVEN,
        grid_position_x: int | NotGiven = NOT_GIVEN,
        grid_position_y: int | NotGiven = NOT_GIVEN,
        prompt: str | NotGiven = NOT_GIVEN,
        resized_height: int | NotGiven = NOT_GIVEN,
        resized_width: int | NotGiven = NOT_GIVEN,
        x_end: int | NotGiven = NOT_GIVEN,
        x_start: int | NotGiven = NOT_GIVEN,
        y_end: int | NotGiven = NOT_GIVEN,
        y_start: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Generation:
        """
        Reframe an image by its ID

        Args:
          aspect_ratio: The aspect ratio of the generation

          media: The image entity object

          model: The model used for the reframe image

          callback_url: The callback URL of the generation, a POST request with Generation object will
              be sent to the callback URL when the generation is dreaming, completed, or
              failed

          format: The format of the image

          grid_position_x: The x position of the image in the grid

          grid_position_y: The y position of the image in the grid

          prompt: The prompt of the generation

          resized_height: Resized height of source image

          resized_width: Resized width of source image

          x_end: The x end of the crop bounds

          x_start: The x start of the crop bounds

          y_end: The y end of the crop bounds

          y_start: The y start of the crop bounds

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/generations/image/reframe",
            body=maybe_transform(
                {
                    "aspect_ratio": aspect_ratio,
                    "generation_type": generation_type,
                    "media": media,
                    "model": model,
                    "callback_url": callback_url,
                    "format": format,
                    "grid_position_x": grid_position_x,
                    "grid_position_y": grid_position_y,
                    "prompt": prompt,
                    "resized_height": resized_height,
                    "resized_width": resized_width,
                    "x_end": x_end,
                    "x_start": x_start,
                    "y_end": y_end,
                    "y_start": y_start,
                },
                image_reframe_params.ImageReframeParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Generation,
        )


class AsyncImageResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncImageResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/lumalabs/lumaai-python#accessing-raw-response-data-eg-headers
        """
        return AsyncImageResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncImageResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/lumalabs/lumaai-python#with_streaming_response
        """
        return AsyncImageResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        model: Literal["photon-1", "photon-flash-1"],
        aspect_ratio: Literal["1:1", "16:9", "9:16", "4:3", "3:4", "21:9", "9:21"] | NotGiven = NOT_GIVEN,
        callback_url: str | NotGiven = NOT_GIVEN,
        character_ref: image_create_params.CharacterRef | NotGiven = NOT_GIVEN,
        format: Literal["jpg", "png"] | NotGiven = NOT_GIVEN,
        generation_type: Literal["image"] | NotGiven = NOT_GIVEN,
        image_ref: Iterable[image_create_params.ImageRef] | NotGiven = NOT_GIVEN,
        modify_image_ref: image_create_params.ModifyImageRef | NotGiven = NOT_GIVEN,
        prompt: str | NotGiven = NOT_GIVEN,
        style_ref: Iterable[image_create_params.StyleRef] | NotGiven = NOT_GIVEN,
        sync: bool | NotGiven = NOT_GIVEN,
        sync_timeout: float | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Generation:
        """
        Generate an image with the provided prompt

        Args:
          model: The model used for the generation

          aspect_ratio: The aspect ratio of the generation

          callback_url: The callback URL for the generation

          format: The format of the image

          modify_image_ref: The modify image reference object

          prompt: The prompt of the generation

          sync: Create image in synchronous mode and return complated image

          sync_timeout: The timeout for the synchronous image generation

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/generations/image",
            body=await async_maybe_transform(
                {
                    "model": model,
                    "aspect_ratio": aspect_ratio,
                    "callback_url": callback_url,
                    "character_ref": character_ref,
                    "format": format,
                    "generation_type": generation_type,
                    "image_ref": image_ref,
                    "modify_image_ref": modify_image_ref,
                    "prompt": prompt,
                    "style_ref": style_ref,
                    "sync": sync,
                    "sync_timeout": sync_timeout,
                },
                image_create_params.ImageCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Generation,
        )

    async def reframe(
        self,
        *,
        aspect_ratio: Literal["1:1", "16:9", "9:16", "4:3", "3:4", "21:9", "9:21"],
        generation_type: Literal["reframe_image"],
        media: image_reframe_params.Media,
        model: Literal["photon-1", "photon-flash-1"],
        callback_url: str | NotGiven = NOT_GIVEN,
        format: Literal["jpg", "png"] | NotGiven = NOT_GIVEN,
        grid_position_x: int | NotGiven = NOT_GIVEN,
        grid_position_y: int | NotGiven = NOT_GIVEN,
        prompt: str | NotGiven = NOT_GIVEN,
        resized_height: int | NotGiven = NOT_GIVEN,
        resized_width: int | NotGiven = NOT_GIVEN,
        x_end: int | NotGiven = NOT_GIVEN,
        x_start: int | NotGiven = NOT_GIVEN,
        y_end: int | NotGiven = NOT_GIVEN,
        y_start: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Generation:
        """
        Reframe an image by its ID

        Args:
          aspect_ratio: The aspect ratio of the generation

          media: The image entity object

          model: The model used for the reframe image

          callback_url: The callback URL of the generation, a POST request with Generation object will
              be sent to the callback URL when the generation is dreaming, completed, or
              failed

          format: The format of the image

          grid_position_x: The x position of the image in the grid

          grid_position_y: The y position of the image in the grid

          prompt: The prompt of the generation

          resized_height: Resized height of source image

          resized_width: Resized width of source image

          x_end: The x end of the crop bounds

          x_start: The x start of the crop bounds

          y_end: The y end of the crop bounds

          y_start: The y start of the crop bounds

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/generations/image/reframe",
            body=await async_maybe_transform(
                {
                    "aspect_ratio": aspect_ratio,
                    "generation_type": generation_type,
                    "media": media,
                    "model": model,
                    "callback_url": callback_url,
                    "format": format,
                    "grid_position_x": grid_position_x,
                    "grid_position_y": grid_position_y,
                    "prompt": prompt,
                    "resized_height": resized_height,
                    "resized_width": resized_width,
                    "x_end": x_end,
                    "x_start": x_start,
                    "y_end": y_end,
                    "y_start": y_start,
                },
                image_reframe_params.ImageReframeParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Generation,
        )


class ImageResourceWithRawResponse:
    def __init__(self, image: ImageResource) -> None:
        self._image = image

        self.create = to_raw_response_wrapper(
            image.create,
        )
        self.reframe = to_raw_response_wrapper(
            image.reframe,
        )


class AsyncImageResourceWithRawResponse:
    def __init__(self, image: AsyncImageResource) -> None:
        self._image = image

        self.create = async_to_raw_response_wrapper(
            image.create,
        )
        self.reframe = async_to_raw_response_wrapper(
            image.reframe,
        )


class ImageResourceWithStreamingResponse:
    def __init__(self, image: ImageResource) -> None:
        self._image = image

        self.create = to_streamed_response_wrapper(
            image.create,
        )
        self.reframe = to_streamed_response_wrapper(
            image.reframe,
        )


class AsyncImageResourceWithStreamingResponse:
    def __init__(self, image: AsyncImageResource) -> None:
        self._image = image

        self.create = async_to_streamed_response_wrapper(
            image.create,
        )
        self.reframe = async_to_streamed_response_wrapper(
            image.reframe,
        )
